]> sigrok.org Git - libsigrok.git/blobdiff - strutil.c
Use g_try_malloc/g_free/g_strdup consistently.
[libsigrok.git] / strutil.c
index 955adedbe5863041fea9cbe662b57f0527352346..5e8689e7a17768fc5db21ce5f80e6323edb3b97f 100644 (file)
--- a/strutil.c
+++ b/strutil.c
@@ -21,7 +21,8 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sigrok.h>
+#include "sigrok.h"
+#include "sigrok-internal.h"
 
 /**
  * Convert a numeric samplerate value to its "natural" string representation.
  * E.g. a value of 3000000 would be converted to "3 MHz", 20000 to "20 kHz".
  *
  * @param samplerate The samplerate in Hz.
* @return A malloc()ed string representation of the samplerate value,
*         or NULL upon errors. The caller is responsible to free() the memory.
  */
-char *sr_samplerate_string(uint64_t samplerate)
+SR_API char *sr_samplerate_string(uint64_t samplerate)
 {
        char *o;
        int r;
 
-       o = malloc(30 + 1); /* Enough for a uint64_t as string + " GHz". */
-       if (!o)
+       /* Allocate enough for a uint64_t as string + " GHz". */
+       if (!(o = g_try_malloc0(30 + 1))) {
+               sr_err("strutil: %s: o malloc failed", __func__);
                return NULL;
+       }
 
-       if (samplerate >= GHZ(1))
+       if (samplerate >= SR_GHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " GHz", samplerate / 1000000000);
-       else if (samplerate >= MHZ(1))
+       else if (samplerate >= SR_MHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " MHz", samplerate / 1000000);
-       else if (samplerate >= KHZ(1))
+       else if (samplerate >= SR_KHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " kHz", samplerate / 1000);
        else
                r = snprintf(o, 30, "%" PRIu64 " Hz", samplerate);
 
        if (r < 0) {
                /* Something went wrong... */
-               free(o);
+               g_free(o);
                return NULL;
        }
 
@@ -60,43 +64,54 @@ char *sr_samplerate_string(uint64_t samplerate)
 }
 
 /**
- * Convert a numeric samplerate value to the "natural" string representation
+ * Convert a numeric frequency value to the "natural" string representation
  * of its period.
  *
  * E.g. a value of 3000000 would be converted to "3 us", 20000 to "50 ms".
  *
  * @param frequency The frequency in Hz.
* @return A malloc()ed string representation of the frequency value,
*         or NULL upon errors. The caller is responsible to free() the memory.
  */
-char *sr_period_string(uint64_t frequency)
+SR_API char *sr_period_string(uint64_t frequency)
 {
        char *o;
        int r;
 
-       o = malloc(30 + 1); /* Enough for a uint64_t as string + " ms". */
-       if (!o)
+       /* Allocate enough for a uint64_t as string + " ms". */
+       if (!(o = g_try_malloc0(30 + 1))) {
+               sr_err("strutil: %s: o malloc failed", __func__);
                return NULL;
+       }
 
-       if (frequency >= GHZ(1))
+       if (frequency >= SR_GHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " ns", frequency / 1000000000);
-       else if (frequency >= MHZ(1))
+       else if (frequency >= SR_MHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " us", frequency / 1000000);
-       else if (frequency >= KHZ(1))
+       else if (frequency >= SR_KHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " ms", frequency / 1000);
        else
                r = snprintf(o, 30, "%" PRIu64 " s", frequency);
 
        if (r < 0) {
                /* Something went wrong... */
-               free(o);
+               g_free(o);
                return NULL;
        }
 
        return o;
 }
 
-char **sr_parse_triggerstring(struct sr_device *device, const char *triggerstring)
+/**
+ * TODO
+ *
+ * @param device TODO
+ * @param triggerstring TODO
+ * @return TODO
+ */
+SR_API char **sr_parse_triggerstring(struct sr_device *device,
+                                    const char *triggerstring)
 {
        GSList *l;
        struct sr_probe *probe;
@@ -106,7 +121,12 @@ char **sr_parse_triggerstring(struct sr_device *device, const char *triggerstrin
 
        max_probes = g_slist_length(device->probes);
        error = FALSE;
-       triggerlist = g_malloc0(max_probes * sizeof(char *));
+
+       if (!(triggerlist = g_try_malloc0(max_probes * sizeof(char *)))) {
+               sr_err("session file: %s: metafile malloc failed", __func__);
+               return NULL;
+       }
+
        tokens = g_strsplit(triggerstring, ",", max_probes);
        trigger_types = device->plugin->get_device_info(0, SR_DI_TRIGGER_TYPES);
        if (trigger_types == NULL)
@@ -130,7 +150,7 @@ char **sr_parse_triggerstring(struct sr_device *device, const char *triggerstrin
                }
 
                if (probenum < 1 || probenum > max_probes) {
-                       printf("Invalid probe.\n");
+                       sr_err("Invalid probe.\n");
                        error = TRUE;
                        break;
                }
@@ -138,7 +158,7 @@ char **sr_parse_triggerstring(struct sr_device *device, const char *triggerstrin
                if ((trigger = strchr(tokens[i], '='))) {
                        for (tc = ++trigger; *tc; tc++) {
                                if (strchr(trigger_types, *tc) == NULL) {
-                                       printf("Unsupported trigger type "
+                                       sr_err("Unsupported trigger type "
                                               "'%c'\n", *tc);
                                        error = TRUE;
                                        break;
@@ -152,8 +172,7 @@ char **sr_parse_triggerstring(struct sr_device *device, const char *triggerstrin
 
        if (error) {
                for (i = 0; i < max_probes; i++)
-                       if (triggerlist[i])
-                               g_free(triggerlist[i]);
+                       g_free(triggerlist[i]);
                g_free(triggerlist);
                triggerlist = NULL;
        }
@@ -161,43 +180,80 @@ char **sr_parse_triggerstring(struct sr_device *device, const char *triggerstrin
        return triggerlist;
 }
 
-uint64_t sr_parse_sizestring(const char *sizestring)
+/**
+ * Convert a "natural" string representation of a size value to uint64_t.
+ *
+ * E.g. a value of "3k" or "3 K" would be converted to 3000, a value
+ * of "15M" would be converted to 15000000.
+ *
+ * Value representations other than decimal (such as hex or octal) are not
+ * supported. Only 'k' (kilo), 'm' (mega), 'g' (giga) suffixes are supported.
+ * Spaces (but not other whitespace) between value and suffix are allowed.
+ *
+ * @param sizestring A string containing a (decimal) size value.
+ * @param size Pointer to uint64_t which will contain the string's size value.
+ * @return SR_OK or error code
+ *
+ */
+SR_API int sr_parse_sizestring(const char *sizestring, uint64_t *size)
 {
-       int multiplier;
-       uint64_t val;
+       int multiplier, done;
        char *s;
 
-       val = strtoull(sizestring, &s, 10);
+       *size = strtoull(sizestring, &s, 10);
        multiplier = 0;
-       while (s && *s && multiplier == 0) {
+       done = FALSE;
+       while (s && *s && multiplier == 0 && !done) {
                switch (*s) {
                case ' ':
                        break;
                case 'k':
                case 'K':
-                       multiplier = KHZ(1);
+                       multiplier = SR_KHZ(1);
                        break;
                case 'm':
                case 'M':
-                       multiplier = MHZ(1);
+                       multiplier = SR_MHZ(1);
                        break;
                case 'g':
                case 'G':
-                       multiplier = GHZ(1);
+                       multiplier = SR_GHZ(1);
                        break;
                default:
-                       val = 0;
-                       multiplier = -1;
+                       done = TRUE;
+                       s--;
                }
                s++;
        }
        if (multiplier > 0)
-               val *= multiplier;
+               *size *= multiplier;
 
-       return val;
+       if (*s && strcasecmp(s, "Hz"))
+               return SR_ERR;
+
+       return SR_OK;
 }
 
-uint64_t sr_parse_timestring(const char *timestring)
+/**
+ * Convert a "natural" string representation of a time value to an
+ * uint64_t value in milliseconds.
+ *
+ * E.g. a value of "3s" or "3 s" would be converted to 3000, a value
+ * of "15ms" would be converted to 15.
+ *
+ * Value representations other than decimal (such as hex or octal) are not
+ * supported. Only lower-case "s" and "ms" time suffixes are supported.
+ * Spaces (but not other whitespace) between value and suffix are allowed.
+ *
+ * @param timestring A string containing a (decimal) time value.
+ * @return The string's time value as uint64_t, in milliseconds.
+ *
+ * TODO: Error handling.
+ * TODO: Add support for "m" (minutes) and others.
+ * TODO: picoseconds?
+ * TODO: Allow both lower-case and upper-case.
+ */
+SR_API uint64_t sr_parse_timestring(const char *timestring)
 {
        uint64_t time_msec;
        char *s;
@@ -220,3 +276,16 @@ uint64_t sr_parse_timestring(const char *timestring)
        return time_msec;
 }
 
+SR_API gboolean sr_parse_boolstring(const char *boolstr)
+{
+       if (!boolstr)
+               return FALSE;
+
+       if (!g_strcasecmp(boolstr, "true") ||
+           !g_strcasecmp(boolstr, "yes") ||
+           !g_strcasecmp(boolstr, "on") ||
+           !g_strcasecmp(boolstr, "1"))
+               return TRUE;
+
+       return FALSE;
+}
@@ -21,7 +21,8 @@
  *
  * @param samplerate The samplerate in Hz.
- * @return A malloc()ed string representation of the samplerate value,
- *         or NULL upon errors. The caller is responsible to free() the memory.
+ * @return A g_try_malloc()ed string representation of the samplerate value,
+ *         or NULL upon errors. The caller is responsible to g_free() the
+ *         memory.
  */
@@ -60,43 +64,54 @@ char *sr_samplerate_string(uint64_t samplerate)
  *
  * @param frequency The frequency in Hz.
- * @return A malloc()ed string representation of the frequency value,
- *         or NULL upon errors. The caller is responsible to free() the memory.
+ * @return A g_try_malloc()ed string representation of the frequency value,
+ *         or NULL upon errors. The caller is responsible to g_free() the
+ *         memory.
  */